#!/usr/bin/env node
// cli.js
import { Command } from 'commander';
import { runTextify } from './lib/main.js';
import { createRequire } from 'module';

const require = createRequire(import.meta.url);
const pkg = require('./package.json'); // Load package.json for version

const program = new Command();

program
  .version(pkg.version)
  .description(pkg.description)
  .argument('[directory]', 'The directory to process', '.') // Default to current directory
  .option('-o, --output <file>', 'Specify output file path (default: stdout)')
  .option('-i, --ignore <pattern...>', 'Add custom ignore patterns (repeatable)') // Use '...' for repeatable options
  .option('-d, --max-depth <depth>', 'Maximum depth for directory tree', (value) => {
    const parsed = parseInt(value, 10);
    if (isNaN(parsed) || parsed < 1) {
      throw new program.InvalidArgumentError('Max depth must be a positive integer.'); // Use program.InvalidArgumentError
    }
    return parsed;
  }, 5)
  .option('--no-gitignore', 'Disable reading .gitignore files')
  .option('--no-default-ignore', 'Disable built-in default ignore patterns')
  .option('--scrub', 'Enable basic secret scrubbing (use with caution!)')
  .option('--encoding <encoding>', 'Specify file reading encoding', 'utf-8')
  .option('--tree-only', 'Only output the directory tree structure')
  .option('--content-only', 'Only output the file contents (no tree)')
  .action(async (directory, options) => {
    const textifyOptions = {
      directory,
      output: options.output,
      ignorePatterns: options.ignore || [], // Ensure it's an array
      maxDepth: options.maxDepth,
      useGitignore: options.gitignore, // Commander makes --no-X options available as false
      useDefaultIgnore: options.defaultIgnore,
      scrub: options.scrub || false,
      encoding: options.encoding,
      treeOnly: options.treeOnly || false,
      contentOnly: options.contentOnly || false,
    };

    if (textifyOptions.treeOnly && textifyOptions.contentOnly) {
        console.error("Error: Cannot use --tree-only and --content-only together.");
        process.exit(1);
    }

    try {
      await runTextify(textifyOptions);
    } catch (error) {
      console.error("\nAn unexpected error occurred:", error);
      process.exit(1);
    }
  });

program.parse(process.argv);