// lib/main.js
import fs from 'fs/promises';
import fssync from 'fs'; // For createWriteStream
import path from 'path';
import ignore from 'ignore';
import chalk from 'chalk';
import { generateTree } from './tree.js';
import { scrubContent, logScrubbingDisclaimer } from './scrub.js';

const DEFAULT_IGNORES = [
    // Core ignores
    'node_modules',
    '.git',
    // Build artifacts
    'dist',
    'build',
    'out',
    'coverage',
    '.next',
    '.nuxt',
    '.svelte-kit',
    // Logs & Lock files
    '*.log',
    'npm-debug.log*',
    'yarn-debug.log*',
    'yarn-error.log*',
    'pnpm-debug.log*',
    'package-lock.json',
    'yarn.lock',
    'pnpm-lock.yaml',
    // Environment & Secrets (KEEP THESE!)
    '.env',
    '.env.*',
    '!.env.example',
    '!.env.sample',
    'secrets.yml',
    '*.pem',
    '*.key',
    // OS specific
    '.DS_Store',
    'Thumbs.db',
    // Common binary file extensions
    '*.png', '*.jpg', '*.jpeg', '*.gif', '*.bmp', '*.tiff', '*.ico', '*.webp',
    '*.pdf', '*.doc', '*.docx', '*.xls', '*.xlsx', '*.ppt', '*.pptx',
    '*.zip', '*.tar', '*.gz', '*.rar', '*.7z', '*.tgz', '*.bz2',
    '*.mp3', '*.wav', '*.ogg', '*.aac', '*.flac',
    '*.mp4', '*.avi', '*.mov', '*.wmv', '*.mkv', '*.webm',
    '*.exe', '*.dll', '*.so', '*.dylib', '*.app', '*.dmg', '*.pkg',
    '*.o', '*.a', '*.obj', '*.lib', '*.wasm',
    '*.pyc', '*.pyo', '*.pyd',
    '*.class', '*.jar',
    '*.sqlite', '*.sqlite3', '*.db', '*.db3', '*.sqlitedb',
    '*.woff', '*.woff2', '*.eot', '*.ttf', '*.otf',
    // IDE/Editor specific
    '.vscode',
    '.idea',
    '*.swp',
    '*.swo',
];

async function findGitIgnore(startDir) {
    let currentDir = path.resolve(startDir);
    const root = path.parse(currentDir).root;
    while (currentDir !== root) {
        const gitignorePath = path.join(currentDir, '.gitignore');
        try {
            await fs.access(gitignorePath);
            return gitignorePath;
        } catch {
            // Not found, go up
        }
        const parentDir = path.dirname(currentDir);
        if (parentDir === currentDir) break; // Safety break
        currentDir = parentDir;
    }
    return null; // Not found
}

// Restored walkDir without debug logs
async function* walkDir(dir, ignoreInstance, rootDir, currentDepth = 1) {
    let entries;
    try {
        entries = await fs.readdir(dir, { withFileTypes: true });
    } catch (err) {
        if (err.code === 'EACCES' || err.code === 'EPERM') {
           console.warn(chalk.yellow(`   Skipping inaccessible directory: ${path.relative(rootDir, dir)}`));
        } else {
           console.warn(chalk.yellow(`   Warning: Could not read directory ${path.relative(rootDir, dir)}: ${err.message}`));
        }
        return; // Stop descent
    }

    for (const entry of entries) {
        const entryPath = path.join(dir, entry.name);
        const relativePath = path.relative(rootDir, entryPath).split(path.sep).join(path.posix.sep);
        const relativePathWithSlash = entry.isDirectory() ? relativePath + '/' : relativePath;

        // Test against ignore rules
        if (ignoreInstance.ignores(relativePathWithSlash) || ignoreInstance.ignores(entry.name)) {
            continue;
        }

        if (entry.isDirectory()) {
            yield* walkDir(entryPath, ignoreInstance, rootDir, currentDepth + 1);
        } else if (entry.isFile()) {
            yield { path: entryPath, relativePath: relativePath };
        }
    }
}


export async function runTextify(options) {
    const { directory, output, ignorePatterns, maxDepth, useGitignore, useDefaultIgnore, scrub, encoding, treeOnly, contentOnly } = options;
    const rootDir = path.resolve(directory);
    let outputStream;
    let outputFilePath = null;

    // --- Validate Root Directory ---
    try {
        const stats = await fs.stat(rootDir);
        if (!stats.isDirectory()) {
            console.error(chalk.red(`Error: The specified path "${directory}" is not a directory.`));
            process.exit(1);
        }
    } catch (err) {
        if (err.code === 'ENOENT') {
            console.error(chalk.red(`Error: The specified directory "${directory}" does not exist.`));
        } else {
            console.error(chalk.red(`Error: Could not access directory "${directory}": ${err.message}`));
        }
        process.exit(1);
    }


    // --- Initialize Ignore Instance ---
    const ig = ignore(); // Create the main ignore instance
    if (useDefaultIgnore) {
        console.log(chalk.blue('Adding default ignore patterns...'));
        ig.add(DEFAULT_IGNORES);
    }

    if (useGitignore) {
        const gitignorePath = await findGitIgnore(rootDir);
        if (gitignorePath) {
            try {
                const gitignoreContent = await fs.readFile(gitignorePath, 'utf-8');
                ig.add(gitignoreContent);
                console.log(chalk.blue(`Loaded rules from ${path.relative(process.cwd(), gitignorePath)}`));
            } catch (err) {
                console.warn(chalk.yellow(`   Warning: Could not read ${path.relative(process.cwd(), gitignorePath)}: ${err.message}`));
            }
        } else {
            console.log(chalk.gray('   No .gitignore file found in project hierarchy.'));
        }
    }

    if (ignorePatterns && ignorePatterns.length > 0) {
        console.log(chalk.blue('Adding custom ignore patterns:'), ignorePatterns);
        ig.add(ignorePatterns);
    }


    // --- Prepare Output Stream ---
    if (output) {
        outputFilePath = path.resolve(output);
        console.log(chalk.blue(`Output will be written to: ${outputFilePath}`));
        try {
            await fs.mkdir(path.dirname(outputFilePath), { recursive: true });
            outputStream = fssync.createWriteStream(outputFilePath, { encoding: encoding });
            outputStream.on('error', (err) => {
                 console.error(chalk.red(`Error writing to output file ${outputFilePath}: ${err.message}`));
                 process.exit(1);
            });
        } catch (err) {
            console.error(chalk.red(`Error creating output file stream for ${outputFilePath}: ${err.message}`));
            process.exit(1);
        }
    } else {
        console.log(chalk.blue('Output will be written to stdout.'));
        outputStream = process.stdout;
        outputStream.end = (cb) => { if (cb) cb(); };
    }

    const writeToOutput = (data) => {
        return new Promise((resolve, reject) => {
            const writeCompleted = (err) => {
                if (err) return reject(err);
                 if (outputStream === process.stdout) {
                    resolve();
                } else if (!needsDrain) {
                     resolve();
                }
            };
            const needsDrain = !outputStream.write(data + '\n', encoding, writeCompleted);
            if (needsDrain && outputStream !== process.stdout) {
                outputStream.once('drain', resolve);
            } else if (!needsDrain && outputStream !== process.stdout) {
                 // Handled by callback
            }
        });
    };


    // --- Generate Directory Tree ---
    if (!contentOnly) {
        // Pass the original 'ig' instance
        const treeString = await generateTree(rootDir, ig, maxDepth);
        await writeToOutput(treeString);
        await writeToOutput("\n" + "=".repeat(80) + "\n");
    }

    if (treeOnly) {
        console.log(chalk.green('Tree only mode enabled. Skipping file content.'));
        if (outputStream !== process.stdout) {
           await new Promise(resolve => outputStream.end(resolve));
        }
        console.log(chalk.green('Directory tree generation complete.'));
        return;
    }

    // --- Process Files ---
    if (scrub) {
        logScrubbingDisclaimer();
    }

    console.log(chalk.blue('Scanning and processing files...'));
    let fileCount = 0;
    let filesToProcess = [];
    let ignoredCount = 0;

    // First pass: collect all file paths respecting ignores
    // Pass the original 'ig' instance directly to walkDir
    try {
        for await (const file of walkDir(rootDir, ig, rootDir)) {
            filesToProcess.push(file);
        }
    } catch (err) {
        console.error(chalk.red(`\nError during directory walk: ${err.message}`), err);
        if (outputStream !== process.stdout) outputStream.end();
        process.exit(1);
    }

    // Sort files alphabetically by relative path for consistent output
    filesToProcess.sort((a, b) => a.relativePath.localeCompare(b.relativePath));

    // Second pass: read and write content
    for (const { path: filePath, relativePath } of filesToProcess) {
        try {
            let content = await fs.readFile(filePath, { encoding: encoding });
            fileCount++;

            if (scrub) {
                content = scrubContent(content, relativePath);
            }

            const headerPath = relativePath.split(path.sep).join(path.posix.sep);
            await writeToOutput(`--- START FILE: ${headerPath} ---`);
            await writeToOutput(content);
            await writeToOutput(`--- END FILE: ${headerPath} ---`);
            await writeToOutput('');

        } catch (error) {
             if (error.code === 'ERR_INVALID_ARG_VALUE' || (error instanceof TypeError && error.message.includes('argument must be of type string or an instance of Buffer'))) {
                console.warn(chalk.yellow(`   Skipping binary or non-${encoding} file: ${relativePath}`));
                ignoredCount++;
            } else if (error.code === 'EACCES') {
                console.warn(chalk.yellow(`   Skipping inaccessible file: ${relativePath}`));
                ignoredCount++;
            } else {
                console.warn(chalk.yellow(`   Warning: Could not read or process file ${relativePath}: ${error.message}`));
                ignoredCount++;
            }
        }
    }

    // --- Finalize ---
    const endMessage = `\nProcessed ${fileCount} files${ignoredCount > 0 ? ` (skipped ${ignoredCount})` : ''}.`;
    if (outputStream !== process.stdout) {
         await new Promise(resolve => outputStream.end(() => {
             console.log(chalk.green(`${endMessage} Output written to ${outputFilePath}`));
             resolve();
         }));
    } else {
        console.log(chalk.green(endMessage + ' Output sent to stdout.'));
    }
}