// lib/scrub.js
import chalk from 'chalk';

// VERY basic regex patterns. Expand and refine these carefully!
// Focus on finding common key patterns and replacing the VALUE part.
const secretPatterns = [
  // Common Keys Assignment (e.g., KEY=value, KEY: value)
  {
    name: 'Common Keys Assignment',
    // Matches variations like: KEY=value, KEY = 'value', KEY:"value", KEY : value
    // Avoids matching common JS/TS constructs like object properties or types
    // Uses negative lookarounds to avoid matching inside typical code structures like object literals being assigned
    regex: /(?<![:\w])\b(api_?key|secret|password|token|passwd|pwd|credential|auth_?token)\b\s*([:=])\s*(['"]?)(?!process\.env\.|import\.meta\.env\.|getenv\(|os\.environ\[|config\.[a-zA-Z_]+|[:{,]|\s*<|true|false|null)([^'"\s#;,]+?)\3(\s*[;,]?\s*(?:#.*|\/\/.*)?$)/gmi,
    replacement: (match, key, separator, quote, value, endQuote, commentOrEnd) => {
      const trimmedValue = value.trim();
      // Basic check: avoid scrubbing very short values, placeholders, or numbers
      if (trimmedValue.length > 3 && !/^<.*>$/.test(trimmedValue) && isNaN(trimmedValue) && trimmedValue !== '[SCRUBBED]') {
        console.warn(chalk.yellow(`    Scrubbed potential secret value for key "${key}"`));
        return `${key}${separator}${quote}[SCRUBBED]${quote}${commentOrEnd}`;
      }
      return match; // Don't replace if value is short, looks like a placeholder, is a number, or already scrubbed
    }
  },
   // JSON-like structures: "key": "value" (more specific)
   {
    name: 'JSON Key/Value',
    // Looks for quoted keys followed by colon and quoted values
    regex: /(['"]\b(?:api_?key|secret|password|token|passwd|pwd|credential|auth_?token)\b['"]\s*:\s*)(['"])(?!process\.env\.|import\.meta\.env\.|true|false|null)([^"'\\]*(?:\\.[^"'\\]*)*)\2/gi,
    replacement: (match, keyPart, quote, value) => {
      const trimmedValue = value.trim();
      if (trimmedValue.length > 3 && !/^<.*>$/.test(trimmedValue) && isNaN(trimmedValue) && trimmedValue !== '[SCRUBBED]') {
        console.warn(chalk.yellow(`    Scrubbed potential secret value for JSON key ${keyPart.trim()}`));
        return `${keyPart}${quote}[SCRUBBED]${quote}`;
      }
      return match;
    }
  },
  // URLs with credentials: protocol://user:password@host
   {
    name: 'URL Credentials',
    regex: /(https?|ftp|mongodb(?:\+srv)?):\/\/([^:@/\s]+):([^:@/\s]+)@/gi,
    replacement: (match, protocol, username, password) => {
      // Avoid scrubbing if password looks like a variable placeholder e.g., ${DB_PASS}
      if (password && password !== '[SCRUBBED]' && !/\$\{.*\}/.test(password)) {
        console.warn(chalk.yellow(`    Scrubbed potential password in URL for user "${username}"`));
        return `${protocol}://${username}:[SCRUBBED]@`;
      }
      return match;
    }
   },
  // AWS Keys (Basic Patterns)
  {
    name: 'AWS Access Key ID',
    // Matches AKIA followed by 16 uppercase letters or numbers
    regex: /(['"]?)(AKIA[0-9A-Z]{16})\1/g,
    replacement: (match, quote, value) => {
      console.warn(chalk.yellow(`    Scrubbed potential AWS Access Key ID: ${value.substring(0, 8)}...`));
      return `${quote}[SCRUBBED_AWS_KEY_ID]${quote}`;
    }
  },
  // Example: Google Cloud API Key (AIza...)
  {
    name: 'Google Cloud API Key',
    // Matches AIza followed by 35 alphanumeric chars with hyphens/underscores
    regex: /(['"]?)(AIza[0-9A-Za-z\\-_]{35})\1/g,
    replacement: (match, quote, value) => {
      console.warn(chalk.yellow(`    Scrubbed potential Google Cloud API Key: ${value.substring(0, 8)}...`));
      return `${quote}[SCRUBBED_GCP_API_KEY]${quote}`;
    }
  },
   // Private Key Blocks
   {
    name: 'Private Key Block',
    // Matches standard PEM block markers
    regex: /-----BEGIN ((?:RSA|EC|PGP|OPENSSH) )?PRIVATE KEY-----[\s\S]*?-----END \1?PRIVATE KEY-----/gi,
    replacement: (match, keyType) => {
         const type = keyType ? keyType.trim() : 'Generic';
         console.warn(chalk.yellow(`    Scrubbed block resembling a ${type} Private Key.`));
         return `-----BEGIN ${type} PRIVATE KEY-----\n[SCRUBBED]\n-----END ${type} PRIVATE KEY-----`;
    }
   },
   // Generic High Entropy String (Heuristic - Use with Caution!)
   // Tries to find long (40+) alphanumeric strings with mix of upper/lower/digits
   // Excludes common hex strings (like Git SHAs)
   // {
   //   name: 'Generic High Entropy String',
   //   regex: /(?<![a-zA-Z0-9/+=])([A-Za-z0-9/+=]{40,})(?![a-zA-Z0-9/+=])/g,
   //   replacement: (match, value) => {
   //     // Exclude all-numeric, all-lower-hex, all-upper-hex
   //     if (!/^\d+$/.test(value) && !/^[a-f0-9]+$/.test(value) && !/^[A-F0-9]+$/.test(value) && /[a-z]/.test(value) && /[A-Z]/.test(value) && /[0-9]/.test(value)) {
   //       console.warn(chalk.yellow(`    Potentially scrubbed generic high-entropy string: ${value.substring(0, 8)}...`));
   //       return '[SCRUBBED_HIGH_ENTROPY]';
   //     }
   //     return match;
   //   }
   // }
];

export function scrubContent(content, filename = '') {
  let scrubbedContent = content;
  if (!content || typeof content !== 'string') {
     return content; // Return original if not a string or empty
  }

  for (const pattern of secretPatterns) {
    // Skip potentially problematic patterns by default unless explicitly enabled
    if (pattern.name === 'Generic High Entropy String' && !process.env.TEXTIFY_ENABLE_BROAD_SCRUBBING) {
        continue;
    }

    try {
      // Check if regex exists and has global flag (required for iterative replacement)
      if(pattern.regex instanceof RegExp && pattern.regex.global) {
        // Pass filename to replacement function if it accepts more than standard args
        if (typeof pattern.replacement === 'function') {
            // Use replaceAll with regex for global replacement
            scrubbedContent = scrubbedContent.replaceAll(pattern.regex, (...args) => {
                 // Check arity of replacement function
                 return pattern.replacement.length > 1 ? pattern.replacement(...args, filename) : pattern.replacement(...args);
            });
        } else {
            // Simple string replacement (less common for scrubbing logic)
             scrubbedContent = scrubbedContent.replaceAll(pattern.regex, pattern.replacement);
        }
      } else if (pattern.regex instanceof RegExp && !pattern.regex.global) {
          console.warn(chalk.magenta(`   Scrubbing pattern "${pattern.name}" does not have global flag. May only replace first match.`));
          // Apply non-global regex once
           if (typeof pattern.replacement === 'function') {
                scrubbedContent = scrubbedContent.replace(pattern.regex, (...args) => {
                     return pattern.replacement.length > 1 ? pattern.replacement(...args, filename) : pattern.replacement(...args);
                });
           } else {
                scrubbedContent = scrubbedContent.replace(pattern.regex, pattern.replacement);
           }
      }
    } catch (error) {
        console.error(chalk.red(`Error applying scrub pattern "${pattern.name}" on file "${filename}":`), error);
        // Continue with the next pattern, but keep the partially scrubbed content
    }
  }
  return scrubbedContent;
}

export function logScrubbingDisclaimer() {
    console.log(chalk.bold.yellow('\n--- SECRET SCRUBBING ACTIVATED ---'));
    console.log(chalk.yellow('Performing basic detection and replacement of potential secrets based on common patterns.'));
    console.log(chalk.red.bold('CRITICAL WARNING: Automated scrubbing is heuristic and NOT guaranteed to be complete or accurate.'));
    console.log(chalk.red.bold('It may miss secrets or incorrectly scrub non-secret data (false positives).'));
    console.log(chalk.yellow.bold('>>> ALWAYS MANUALLY REVIEW THE OUTPUT for sensitive data before sharing it anywhere. <<<'));
    console.log(chalk.yellow('--- END SCRUBBING WARNING ---\n'));
}