// lib/tree.js
import fs from 'fs/promises';
import path from 'path';
import chalk from 'chalk';

async function generateTreeRecursive(dir, prefix, ignoreInstance, currentDepth, maxDepth, rootDir) {
    if (currentDepth > maxDepth) {
        return '';
    }

    let treeString = '';
    let entries;
    try {
        entries = await fs.readdir(dir, { withFileTypes: true });
    } catch (err) {
        if (err.code === 'EACCES' || err.code === 'EPERM') {
           console.warn(chalk.yellow(`   Skipping inaccessible directory in tree: ${path.relative(rootDir, dir)}`));
        } else {
           console.warn(chalk.yellow(`   Warning: Could not read directory ${path.relative(rootDir, dir)} for tree: ${err.message}`));
        }
        return ''; // Skip directories we can't read/access
    }

    // Filter entries based on ignore rules *before* sorting and processing
    const filteredEntries = entries.filter(entry => {
        const entryPath = path.join(dir, entry.name);
        // Use POSIX separators for relative paths for consistent ignore matching
        const relativePath = path.relative(rootDir, entryPath).split(path.sep).join(path.posix.sep);
        const relativePathWithSlash = entry.isDirectory() ? relativePath + '/' : relativePath;
        // Check if the path itself or just the name is ignored
        return !(ignoreInstance.ignores(relativePathWithSlash) || ignoreInstance.ignores(entry.name));
    });


    // Sort entries: directories first, then files, alphabetically
    filteredEntries.sort((a, b) => {
        if (a.isDirectory() && !b.isDirectory()) return -1;
        if (!a.isDirectory() && b.isDirectory()) return 1;
        return a.name.localeCompare(b.name);
    });

    for (let i = 0; i < filteredEntries.length; i++) {
        const entry = filteredEntries[i];
        const isLast = i === filteredEntries.length - 1;
        const connector = isLast ? '└── ' : '├── ';
        const entryPath = path.join(dir, entry.name);

        treeString += `${prefix}${connector}${entry.name}\n`;

        if (entry.isDirectory()) {
            const newPrefix = prefix + (isLast ? '    ' : '│   ');
            // Pass rootDir down for correct relative path calculation in nested calls/errors
            treeString += await generateTreeRecursive(entryPath, newPrefix, ignoreInstance, currentDepth + 1, maxDepth, rootDir);
        }
    }
    return treeString;
}

export async function generateTree(rootDir, ignoreInstance, maxDepth) {
    console.log(chalk.blue(`Generating directory tree (max depth: ${maxDepth})...`));
    // Pass rootDir to the recursive function start
    const tree = await generateTreeRecursive(rootDir, '', ignoreInstance, 1, maxDepth, rootDir);
    console.log(chalk.green('Directory tree generated.'));
     // Use POSIX separators for the root display as well
    const displayRoot = path.basename(rootDir).split(path.sep).join(path.posix.sep);
    return `Directory Tree (Root: ${displayRoot}, Max Depth: ${maxDepth}):\n${displayRoot}\n${tree}`;
}